/////////////////////////////////////////////////////////////////////////////
///
/// \file macros.h
///
/// \author Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
///
/// \brief Header file that defines all macros definitions
///
/// \version 1.02
///
/// \date September 2008
///
///
/// This file will redefines all macros used in this software like realloc,
/// free, max, min, the score function etc.
///
///
/////////////////////////////////////////////////////////////////////////////


#ifndef __MACROS_H__
#define __MACROS_H__

/*Basic inclusions*/
#include <stdlib.h>
#include <assert.h>
#include <string.h>


/******************************************************************************
 **   Redefine the calloc, realloc and free functions
 **     
 *****************************************************************************/


////////////////////////////////////////////////////////////////////////////
///
/// \def NEW(Ref, Type, Nb)
///
/// \brief Redefine the calloc function
///
/// \param[in,out] Ref : The pointer that needs to be allocated
/// \param[in] Type : Type of the pointer
/// \param[in] Nb : size that needs to be allocated
///
/// This definition defines the calloc function on the pointer "Ref" given 
/// of type "Type" and size "Nb" linked with an assert function
///
/////////////////////////////////////////////////////////////////////////////


#define NEW(Ref, Type, Nb) \
  Ref = (Type*)calloc((Nb), sizeof(Type));	\
  assert(Ref != NULL)


////////////////////////////////////////////////////////////////////////////
///
/// \def RENEW(Ref, Type, Nb)
///
/// \brief Redefine the realloc function
///
/// \param[in,out] Ref : The pointer that needs to be reallocated
/// \param[in] Type : Type of the pointer
/// \param[in] Nb : Total size that needs to be reallocated
///
/// This definition defines the realloc function on the pointer "Ref" given 
/// of type "Type" and size "Nb" linked with an assert function
///
/////////////////////////////////////////////////////////////////////////////


#define RENEW(Ref, Type, Nb) \
  Ref = (Type*)realloc(Ref, (Nb) * sizeof(Type)); \
  assert(Ref != NULL)



////////////////////////////////////////////////////////////////////////////
///
/// \def DESTROY(Ref)
///
/// \brief Redefine the free function
///
/// \param[in,out] Ref : The pointer that needs to be destroyed
///
/// This definition defines the free function on the pointer "Ref" given
///
/////////////////////////////////////////////////////////////////////////////


#define DESTROY(Ref) \
  free(Ref); \
  Ref = NULL



/******************************************************************************
 **   Define the MIN, MAX functions
 **     
 *****************************************************************************/


////////////////////////////////////////////////////////////////////////////
///
/// \def MAX(Aa, Bb)
///
/// \brief Defines a maximum function between 2 numbers
///
/// \param[in] Aa : First number to compare
/// \param[in] Bb : Second number to compare
///
/// \return The max number between the 2 numbers
///
/// This definition defines a function that will determine the maximum
/// number between two numbers
///
/////////////////////////////////////////////////////////////////////////////


#define MAX(Aa, Bb) \
  ((Aa > Bb) ? (Aa) : (Bb))


///////////////////////////////////////////////////////////////////////////
///
/// \def MIN(Aa, Bb)
///
/// \brief Defines a minimum function between 2 numbers
///
/// \param[in] Aa : First number to compare
/// \param[in] Bb : Second number to compare
///
/// \return The min number between the 2 numbers
///
/// This definition defines a function that will determine the minimum
/// number between two numbers
///
///////////////////////////////////////////////////////////////////////////


#define MIN(Aa, Bb) \
  ((Aa < Bb) ? (Aa) : (Bb))



////////////////////////////////////////////////////////////////////////////
///
/// \def MAX3(Aa, Bb)
///
/// \brief Defines a maximum function between 3 numbers
///
/// \param[in] Aa : First number to compare
/// \param[in] Bb : Second number to compare
/// \param{in] Cc : Third number to compare
///
/// \return The max number between the 3 numbers
///
/// This definition defines a function that will determine the maximum
/// number between three numbers
///
/////////////////////////////////////////////////////////////////////////////
	    

#define MAX3(Aa, Bb, Cc)			\
  ((Aa < Bb) ?					\
   ((Cc < Bb) ? (Bb) : (Cc))			\
   :						\
   ((Cc < Aa) ? (Aa) : (Cc)))

#endif
