////////////////////////////////////////////////////////////////////////////
///
/// \file sequence.h
///
/// \author Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
///
/// \brief Header file that defines all functions related to sequences
///
/// \version 1.02
///
/// \date September 2008
///
///
/// This file will define all fucntions related to sequences  
///
///
/////////////////////////////////////////////////////////////////////////////


#ifndef __SEQUENCE_H__
#define __SEQUENCE_H__


/*Intern inclusion*/
#include "types.h"


/////////////////////////////////////////////////////////////////////////////
///
/// \fn void get_sequences(sequence_array *array_of_seqs)
///
/// \brief Function used to retrieve sequences contained in the fasta file
///
/// \param[in,out] array_of_seqs : the array of sequences that will be
/// updated with sequences contained in the fasta file
///
///
/// This function will use the sequence parser to retrieve the sequences in
/// the fasta file
///
///
/////////////////////////////////////////////////////////////////////////////

extern void get_sequences(sequence_array *array_of_seqs);



/////////////////////////////////////////////////////////////////////////////
///
/// \fn void destroy_sequence(p_sequence s)
///
/// \brief Function used to destroy a p_sequence structure
///
/// \param[in,out] s : the p_sequence to destroy
///
///
/// This function will destroy all allocated memory for a p_sequence 
/// structure
///
///
/////////////////////////////////////////////////////////////////////////////

extern void destroy_sequence(p_sequence s);



/////////////////////////////////////////////////////////////////////////////
///
/// \fn int is_present_array_sequence(char *seq_name_to_find,
///                                   sequence_array *array_of_seqs)
///
/// \brief Function used to retrieve the index of a sequence according to
/// its name
///
/// \param[in] seq_name_to_find : the name to search for
/// \param[in] array_of_seqs : the array of sequences
///
/// \return the index in the sequence array or -1 if it wasn't found
///
/// This function will search through the array of sequences for the name and 
/// return its idnex or -1 if it wasn't found
///
///
/////////////////////////////////////////////////////////////////////////////


extern int is_present_array_sequence(char *seq_name_to_find, 
				     sequence_array *array_of_seqs);




/////////////////////////////////////////////////////////////////////////////
///
/// \fn void get_real_length_sequence(p_sequence seq)
///
/// \brief Function used to calculate the real length of a sequence
///
/// \param[in,out] seq : the reference sequence
///
/// This function will calculate the real length of a sequence(all 
/// nucleotides except N)
///
///
/////////////////////////////////////////////////////////////////////////////


extern void get_real_length_sequence(p_sequence seq);



/////////////////////////////////////////////////////////////////////////////
///
/// \fn char get_complementary_base(char nucleotide)
///
/// \brief Function used to get the complementary nucleotide
///
/// \param[in] nucleotide : the nucleotide to get its complementary
///
/// \return the complementary nucleotide
///
/// This function will return the complementary nucleotide of the one given
///
///
/////////////////////////////////////////////////////////////////////////////

extern char get_complementary_base(char nucleotide);


/////////////////////////////////////////////////////////////////////////////
///
/// \fn void complete_sequences_attributes(sequence_array *array_of_seqs)
///
/// \brief Function used to complete some missing attributes in the sequence
/// array
///
/// \param[in,out] : array_of_seqs : the array containing all sequences
/// and that will be updated
///
/// This function will complete the species attribute when possible, put in 
/// uppercase the sequence when needed and replace the spaces in the sequence
/// name by a "_"
///
/////////////////////////////////////////////////////////////////////////////

extern void complete_sequences_attributes(sequence_array *array_of_seqs);


#endif
