/******************************************************************************
 **  
 **  \file sequence_lexer.l
 **  
 **
 ** \author Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
 **
 ** \brief Lexer file that defines the lexical analysor
 **
 ** \version 1.02
 **
 ** \date September 2008
 **
 **
 ** This file will define all lexical terms to recognized a fasta format
 **
 **
 **
 *****************************************************************************/


%{

#include "sequence_interface.h"
#include "sequence_parser.h"
#include "macros.h"
#include <string.h>
#include <assert.h>
%}


%option yylineno
%option never-interactive
%option nounput



SPACE           [[:blank:]]
SPACES          ({SPACE}+)
CRLF            ([\r]?[\n])
CRLFN           ({CRLF}+)
FASTA_TITLE     ^{SPACE}*[>]
NUMBER_ELEMENT   ([[:digit:]]+)
SEQUENCE_ELEMENT ([[:alpha:]*.\-]{1,1000})
ID               ([^\t\r\n]+)
IGNORE_LINE     ^[#].*{CRLFN}
%x FASTA_TITLE_STATE_ID




%%

<INITIAL>{IGNORE_LINE} { }

<INITIAL>{FASTA_TITLE} {
  BEGIN(FASTA_TITLE_STATE_ID);
  return FASTA_TITLE_T;
}


<FASTA_TITLE_STATE_ID>{ID} {
  int nbc;
  nbc = sequenceleng;
  NEW(sequencelval.text, char, (1 + nbc));
  strncpy(sequencelval.text, sequencetext, nbc);
  sequencelval.text[nbc] = '\0';
  return FASTA_NAME_T;
}

<FASTA_TITLE_STATE_ID>{SPACES} {}

<FASTA_TITLE_STATE_ID>{CRLFN} {
  BEGIN(INITIAL);
}

<INITIAL>{SEQUENCE_ELEMENT} {
  NEW(sequencelval.text, char, (1 + sequenceleng));
  strncpy(sequencelval.text, sequencetext, sequenceleng);
  sequencelval.text[sequenceleng] = '\0';
  return SEQUENCE_ELEMENT_T;
}

<INITIAL>{NUMBER_ELEMENT} {}


<INITIAL>{CRLFN} {}
<INITIAL>{SPACES} {}


. { 
  sequenceerror("invalid token"); 
  yyterminate();

}






%%

int sequencewrap() {
  BEGIN(INITIAL);
  return 1;
}




void free_flex_mem() {
  /* Pop the buffer stack, destroying each element. */
  while (YY_CURRENT_BUFFER){
    yy_delete_buffer (YY_CURRENT_BUFFER );
    YY_CURRENT_BUFFER_LVALUE = NULL;
    yypop_buffer_state ();
  }
    
  /* Destroy the stack itself. */
  yyfree ( yy_buffer_stack );
  yy_buffer_stack = NULL;
    
}  /* For non-reentrant C scanner only. */
